//*************************************************
// CORSO PROGRAMMAZIONE PICMICRO
// appendice 2
// Utilizzo degli Shift Register SIPO per aumentare
// il numero di uscite di un picmicro
// www.settorezero.com
//
// modulo: main.c
// autore: Bernardo Giovanni
// data: 26/03/10
// descrizione: test SIPO 74LS164
// picmicro: PIC16F877A
// clock: 20MHz
//
//*************************************************
#define  XTAL_FREQ 20MHZ
#include <pic.h>
#include "delay.c"

__CONFIG (HS & WDTDIS & PWRTEN & BORDIS & LVPDIS & DUNPROT & WRTEN & DEBUGDIS & UNPROTECT);

#define DATA RC7 // pin utilizzato per inviare il dato seriale
#define CK   RC6 // pin utilizzato per il clock

// prototipi funzioni
void main(void);
void sipo_out(char dat); // funzione per invio seriale al SIPO 74LS164

void main(void)
	{
	// Tutte le porte come output 
	TRISA=0;
	TRISB=0;
	TRISC=0;
	TRISD=0;
	TRISE=0;
	
	// Data e Clock devono essere inizializzati a zero
	DATA=0;
	CK=0;
		
	unsigned int a=0; // counter
	for (a=0; a<256; a++)
		{
		sipo_out(a); // invio il valore attuale di "a" al SIPO
		DelayMs(100); // ritardo per consentire la visualizzazione
		}
		
	while(1)
		{continue;} // ciclo infinito
	}

// Questa funzione serializza il byte DAT per inviarlo al SIPO 74LS164
// ma in maniera inversa: il bit 0 del mio dato finir nell'uscita Q7
void sipo_out(unsigned char dat)
	{
	static unsigned char shift=0; // posizioni di shift

	for (shift=0; shift<8; shift++)
		{
		// sposto il byte passato di "shift" posizioni
		// in maniera tale da avere il bit "shift-esimo" in posizione zero
		
		if ((dat >> shift) & 1) // controllo se il bit in posizione zero vale 1 o 0
			{
			DATA=1; // pin data a livello alto se il bit valeva 1
			}
		else
			{
			DATA=0; // pin data a livello basso se il bit valeva 0
			}
		// Invio quindi il segnale di clock:
		// La transizione del clock da basso ad alto sposta il dato nello shift register
		CK=1;
		CK=0;
		}
	}